@@ -384,6 +384,10 @@ def graph_json_values(df_graph, df_basic_data, node_list_df, from_verified):
     if (from_verified == False):            
         df_basic_data = df_basic_data[df_basic_data["verified"] != True].reset_index()
 
+    node_list_verified_filter = list(df_basic_data["screen_name"])
+    node_list_verified_filter_df= pd.DataFrame()
+    node_list_verified_filter_df["nodes"]= node_list_verified_filter  
+
     #Removing incorrect profile mentions
     tweets_profiles =  list(df_basic_data["screen_name"])
     node_list = list(node_list_df["nodes"])
@@ -395,6 +399,8 @@ def graph_json_values(df_graph, df_basic_data, node_list_df, from_verified):
 
     df_graph = df_graph[~df_graph["screen_name"].isin(delete_profiles)]
     df_graph = df_graph[~df_graph["screen_name_mention"].isin(delete_profiles)]    
+
+    
     
     data = {}
     data['nodes'] = []
@@ -419,7 +425,7 @@ def graph_json_values(df_graph, df_basic_data, node_list_df, from_verified):
               'iteration': str(df_graph["iteration"].iloc[i])
             })
       
-    return json.dumps(data)
+    return json.dumps(data), node_list_verified_filter_df
 
 def simple_user_timeline_extraction(node_list, consumerKey, consumerSecret,accessToken,accessTokenSecret,filepath, filename,n_groups=1,save_by=300):
     # get_user Returns the 20 most recent statuses posted from the authenticating user or the user specified. 
@@ -1634,4 +1640,98 @@ def distribution_sentiment_terrorist_tweets(tweets_df):
     plt.show()
 
   
-  
\ No newline at end of file
+ ###---------------------------------------------Telegram Functions -----------------------------------###
+
+def load_telegram_file(json_filename):
+    data = json.load(json_filename)
+    df = pd.DataFrame(data)
+    return df
+
+def nodes_participants_dict(participants_df):
+    l=[]
+    for i in range(len(participants_df)):
+        if str(participants_df.last_name.iloc[i]) != 'None':
+            name = str(participants_df.first_name.iloc[i]) +' '+ str(participants_df.last_name.iloc[i])
+        else:
+            name = participants_df.first_name.iloc[i]
+        l.append((participants_df._id.iloc[i], name))
+
+    nodes_participants_dict= dict(l)
+    return nodes_participants_dict
+
+
+def nodes_participants_df(participants_df):
+    l=[]
+    for i in range(len(participants_df)):
+        if str(participants_df.last_name.iloc[i]) != 'None':
+            name = str(participants_df.first_name.iloc[i]) +' '+ str(participants_df.last_name.iloc[i])
+        else:
+            name = participants_df.first_name.iloc[i]
+        l.append((participants_df._id.iloc[i], name))
+    nodes_participants_df = pd.DataFrame(data=l , columns=['_id','name'])
+    return nodes_participants_df
+
+
+
+def messages_connections (msg, node_participants_dict):
+
+    #Añade una variable en un otra variable diccionario
+    l=[]
+    for i in range(len(msg)):
+        l.append(msg.from_id.iloc[i]['user_id'])
+    msg['sender_id'] = l
+    source = []
+    target = []
+    for i in range(len(msg)):
+        if msg.reply_to.iloc[i] != None :
+            replyid = msg.reply_to.iloc[i]['reply_to_msg_id']       
+            source.append( msg.sender_id.iloc[i])
+            target.append(int(msg[msg['_id']==replyid].sender_id))
+    graph_rel = pd.DataFrame()
+    graph_rel ['screen_name'] = source
+    graph_rel ['screen_name_mention'] = target
+
+    s=[]
+    for i in range(len(graph_rel)):        
+        try:
+            s.append([node_participants_dict[graph_rel.screen_name.iloc[i]],node_participants_dict[graph_rel.screen_name_mention.iloc[i]]])
+        except KeyError:
+             print(graph_rel.screen_name_mention.iloc[i], 'this ID has left the group!')
+    df_graph_connections = pd.DataFrame(data=s,columns = ['screen_name','screen_name_mention'])
+    df_graph_connections ['iteration'] = [1 for i in range(len(df_graph_connections))]
+    return df_graph_connections
+
+
+def df_graph_telegram_features (DiG, nodes_participants_df):
+    DiG.remove_nodes_from(['_id','name'])
+    DiG.add_nodes_from(nodes_participants_df['name'])
+    nodes_names= list(DiG.nodes())
+    neighbours_names=[]
+    in_deg = []
+    out_deg = []
+    degree_cent =[]
+    closeness_cent = []
+    betweenness_centrality = []
+    load_centrality =[]
+    eigenvector_centrality =[]
+    for i in range(len(nodes_names)):
+        neighbours_names.append(list(nx.all_neighbors(DiG, node=nodes_names[i])))
+        in_deg.append(DiG.in_degree()[str(nodes_names[i])])
+        out_deg.append(DiG.out_degree()[str(nodes_names[i])])
+        degree_cent.append(nx.degree_centrality(DiG)[str(nodes_names[i])])
+        closeness_cent.append(nx.closeness_centrality(DiG)[str(nodes_names[i])])
+        betweenness_centrality.append(nx.betweenness_centrality(DiG)[str(nodes_names[i])])
+        load_centrality.append(nx.load_centrality(DiG)[str(nodes_names[i])])
+        eigenvector_centrality.append(nx.eigenvector_centrality(DiG)[str(nodes_names[i])])
+    df_graph_features = pd.DataFrame()
+    df_graph_features['screen_name']=nodes_names
+    df_graph_features['in_degree'] = in_deg
+    df_graph_features['out_degree'] = out_deg
+    df_graph_features['degree'] = df_graph_features['in_degree']  + df_graph_features['out_degree']
+    df_graph_features['degree_centrality'] = degree_cent
+    df_graph_features['closeness_centrality'] = closeness_cent
+    df_graph_features['betweenness_centrality'] = betweenness_centrality
+    df_graph_features['load_centrality'] = load_centrality
+    df_graph_features['eigenvector_centrality'] =eigenvector_centrality
+    df_graph_features["neighbours"] = neighbours_names
+    return df_graph_features
